import os
import scanpy as sc
from io import BytesIO
from bokeh.palettes import d3
import json
import base64
import sys
from pathlib import Path
from bokeh.io import curdoc
from bokeh.models import FileInput, Button, TextInput, Div, Select
from bokeh.layouts import row, column

sys.path.append(str(Path(__file__).resolve().parents[1]))
try:
    from source import connection, plot_function
except:
    from scpantheon.source import connection, plot_function

color_list = d3['Category20c'][20]

class new_layout:
    def __init__(self):
        try:
            group, cluster_list = get_attr()
            api = connection()
            to_json = api.get_attributes()
            data_dict = json.loads(to_json)
            group = data_dict['selected_group']            
            cluster_list = list(api.get_group_dict()[group]['class_name'])
        except:
            print('No Group!')

        test_method = Select(title='Choose the method:', options=['t-test','wilcoxon','logreg'], value='t-test')
        rank_n_genes = TextInput(title='Input the num of genes to rank')
        marker = Button(label='Find marker of these clusters', button_type='success')
        marker.on_click(lambda: find_marker(test_method=test_method.value, rank_n_genes=int(rank_n_genes.value)))
        violin_figure = Button(label='Show violin plot', button_type='success')
        violin_figure.on_click(lambda: violin(cluster_list=cluster_list, gene_num=int(rank_n_genes.value)))

        # layout format changed: delete the inner column
        self.marker = column(row(test_method, rank_n_genes),
                            marker,
                            violin_figure)
        # self.marker.name = 'Find_Marker_Gene'

    def add(self):
        return self.marker


def button_disabled(buttons_group):
    for b in buttons_group:
        b.disabled = True

def button_abled(buttons_group):
    for b in buttons_group:
        b.disabled = False


def get_attr():
    global buttons_group

    plot = plot_function()
    buttons_group, b = plot.get_buttons_group()
    button_disabled(buttons_group)
    def next_get():
        api = connection()
        to_json = api.get_attributes()
        data_dict = json.loads(to_json)
        group = data_dict['selected_group']
        cluster_list = api.get_group_dict()[group]
        return group, cluster_list
        button_abled(buttons_group)
    curdoc().add_next_tick_callback(next_get)

def find_marker(test_method='t-test',rank_n_genes=25):
    button_disabled(buttons_group)
    def next_find():
        layout = curdoc().get_model_by_name('Find_Marker_Gene')
        group, cluster_list = get_attr()
        api = connection()
        to_json = api.get_attributes()
        data_dict = json.loads(to_json)
        group = data_dict['selected_group']            
        adata = api.get_anndata()

        sc.tl.rank_genes_groups(adata, group, method=test_method)
        sc.pl.rank_genes_groups(adata, n_genes=rank_n_genes, sharey=False, save='.png')

        name = 'figures/rank_genes_groups_' + group + '.png'
        img = open(name,'rb')
        img_base64 = base64.b64encode(img.read()).decode("ascii")
        div = Div(text="<img src=\'data:image/png;base64,{}\'/>".format(img_base64))
        layout.children.append(div)
        api.set_obsm(adata.obsm)
        button_abled(buttons_group)
    curdoc().add_next_tick_callback(next_find)

def violin(cluster_list,gene_num):
    button_disabled(buttons_group)
    def next_violin():
        layout = curdoc().get_model_by_name('Find_Marker_Gene')
        group, cluster_list = get_attr()
        api = connection()
        to_json = api.get_attributes()
        data_dict = json.loads(to_json)
        group = data_dict['selected_group']            
        adata = api.get_anndata()

        sc.pl.rank_genes_groups_violin(adata, groups=cluster_list[1:], n_genes=gene_num, save='.png')

        img = open('figures/violin.png','rb')
        img_base64 = base64.b64encode(img.read()).decode("ascii")
        div = Div(text="<img src=\'data:image/png;base64,{}\'/>".format(img_base64))
        layout.children.append(div)

        for cluster in cluster_list[1:]:
            name = 'figures/rank_genes_groups_' + group + '_' + cluster + '.png'
            img = open(name,'rb')
            img_base64 = base64.b64encode(img.read()).decode("ascii")
            div = Div(text="<img src=\'data:image/png;base64,{}\'/>".format(img_base64))
            layout.children.append(div)
        button_abled(buttons_group)
    curdoc().add_next_tick_callback(next_violin)


from multiprocessing import freeze_support
freeze_support()

import warnings

from bokeh.server.server import Server
import multiprocessing
import warnings

class InstallWarning(Warning):
    def __init__(self, message):
        self.message = message
    def __str__(self):
        return repr(self.message)

try: 
    from scpantheon import qt, source # import from online
except:
    warnings.warn('YOU HAVE TO INSTALL PyQt5',InstallWarning)

def run():
    global server
    print('Opening Bokeh application on http://localhost:5006/')
    server = Server({'/': source.main},
                    allow_websocket_origin=["localhost:5006"], port=5006, show=False, num_procs=1) 
    server.start()  
    server.io_loop.start()
    server.show()

def app():
    if qt.main() == 'app closed':
        p1.terminate()
        print('app ended')
    

def main():
    global p1
    p1 = multiprocessing.Process(target=run)
    p1.start()
    app()


if __name__ == '__main__':
    main()


import os
import scanpy as sc
from io import BytesIO
from bokeh.palettes import d3
import json
import base64
import sys
from pathlib import Path
from bokeh.io import curdoc
from bokeh.models import FileInput, Button, TextInput, Div, Select
from bokeh.layouts import row, column
import pandas, anndata
import numpy as np

sys.path.append(str(Path(__file__).resolve().parents[1]))
try:
    from source import connection, plot_function
except:
    from scpantheon.source import connection, plot_function

class new_layout:
    def __init__(self):
        global buttons_group
        self.show_button = Button(label='Show Histogram')
        self.show_button.on_click(check_histogram) 
        self.remove_button = Button(label='Remove')
        self.remove_button.on_click(remove)

    def add(self):
        return column(self.show_button, 
                      self.remove_button)

def button_disabled(buttons_group):
    for b in buttons_group:
        b.disabled = True

def button_abled(buttons_group):
    for b in buttons_group:
        b.disabled = False

def check_histogram():
    global buttons_group

    plot = plot_function()
    buttons_group, b = plot.get_buttons_group() # group and the original amount
    button_disabled(buttons_group)
    def next_check():
        global plotlist, p, glylist
        layout = curdoc().get_model_by_name('Check_Histogram')
        change = connection()
        #pdata = change.get_pandata()
        adata = change.get_anndata()
        plot = plot_function()
        p = plot.get_figure()
        x, y = plot.get_x_y()
        glylist = plot.get_glyph_list()

        # get the x and y of plot
        cnt = 0
        for axis in adata.var.index:
            if(x == axis): x = cnt
            if(y == axis): y = cnt
            cnt += 1
        
        def ad_to_pd():
            global pdata
            X = adata.X
            obs_df = adata.obs
            var_df = adata.var
            pdata = pandas.concat([obs_df, var_df, pandas.DataFrame(X.T)], axis=1)
            pdata = pdata.T
        ad_to_pd()

        if len(glylist) <= 1:
            # pdata[self.data_columns[x_init_idx]].describe()
            hist, edges = np.histogram(pdata[x],bins = int(10/0.05),range = [0, 10])
            hist2, edges2 = np.histogram(pdata[y],bins = int(10/0.05),range = [0, 10])
            amount = pandas.DataFrame({'pdata': np.log(hist),'left': edges[:-1],'right': edges[1:]})
            amount2 = pandas.DataFrame({'pdata': np.log(hist2),'left': edges2[:-1],'right': edges2[1:]})
            q1 = p.quad(bottom=0,top=amount['pdata'],left=amount['left'],right=amount['right'],line_color=None,fill_color="#c3f4b2",fill_alpha=0.3,legend_label='qx')
            l1 = p.line(x=np.linspace(0, 10, 200), y=amount['pdata'], line_color="#3333cc", line_width=2, alpha=0.3, legend_label="lx")
            q2 = p.quad(bottom=amount2['left'],top=amount2['right'],left=0,right=amount2['pdata'],line_color=None,fill_color='#FFC125',fill_alpha=0.3,legend_label='qy')
            l2 = p.line(x=amount2['pdata'], y=np.linspace(0, 10, 200), line_color="#ff8888", line_width=2, alpha=0.3, legend_label="ly")
            plotlist = [q1, q2, l1, l2]
            glylist.extend(plotlist)
            p.legend.location = "center_right"
            p.legend.click_policy = "hide"
        else:
            print('Histogram already existed')
        button_abled(buttons_group)
    curdoc().add_next_tick_callback(next_check)


def remove():
    global glylist
    button_disabled(buttons_group)
    def next_remove():
        if len(glylist) > 1:
            for plot in plotlist:
                glylist.remove(plot)
        else:
            print('Histogram already deleted')
        for glygh in plotlist:
            try:
                p.renderers.remove(glygh)
            except: print('no glyph')
            button_abled(buttons_group)
    curdoc().add_next_tick_callback(next_remove)



from bokeh.models import Button
import json
import sys
from pathlib import Path
from bokeh.io import curdoc

sys.path.append(str(Path(__file__).resolve().parents[1]))
try:
    from source import connection, plot_function
except:
    from scpantheon.source import connection, plot_function

class new_layout:
    def __init__(self):
        self.new_button = Button(label='Change Color')
        self.new_button.on_click(change_color) 
    
    def add(self):
        return self.new_button
    

def button_disabled(buttons_group):
    for b in buttons_group:
        b.disabled = True

def button_abled(buttons_group):
    for b in buttons_group:
        b.disabled = False


def change_color(): 
    global buttons_group

    plot = plot_function()
    buttons_group, b = plot.get_buttons_group()
    button_disabled(buttons_group)
    def next_color():
        plot_function().show_checked()
        trans = connection()
        to_json = trans.get_attributes()
        data_dict = json.loads(to_json)
        color = data_dict['selected_color']
        selected_class = data_dict['checked_class']
        group = data_dict['selected_group']
        group_dict = trans.get_group_dict()
        group_dict[group]['color'][[i for i in selected_class]] = color   
        trans.set_group_dict(group_dict)
        indices = data_dict['selected_indices']
        data_color = data_dict['data_color']
        for i in indices:
            data_color[i] = color
        # Save change of data into the Figure
        data_dict['data_color'] = data_color
        trans.set_attributes(data_dict)
        plot_function().change_checkbox_color()
        button_abled(buttons_group)
    curdoc().add_next_tick_callback(next_color)


import os
from pickletools import optimize
import scanpy as sc
from io import BytesIO
from bokeh.palettes import d3
import json
import base64
import sys
from pathlib import Path
from bokeh.io import curdoc
from bokeh.models import FileInput, Button, TextInput, Div, Select, MultiChoice
from bokeh.layouts import row, column

sys.path.append(str(Path(__file__).resolve().parents[1]))
try:
    from source import connection, plot_function
except:
    from scpantheon.source import connection, plot_function

color_list = d3['Category20c'][20]


class new_layout:
    def __init__(self):
        try:
            group, cluster_list = get_attr()
            api = connection()
            # to_json = api.get_attributes()
            # data_dict = json.loads(to_json)
            # group = data_dict['selected_group']            
            # cluster_list = list(api.get_group_dict()[group]['class_name'])
            total_gene = list(api.get_anndata().var_names)
        except:
            print('No Group!')
        method = Select(title='Select method:', options=['wilcoxon','t-test', 'logreg'], value='t-test')
        gene_num = TextInput(title='Input gene nums to show:', value='20')
        rank_genes = Button(label='Compare gene expression to a single cluster', button_type='success')
        rank_genes.on_click(lambda: rank(method=method.value,gene_num=int(gene_num.value)))
        violin_figure = Button(label='Show violin plot', button_type='success')
        violin_figure.on_click(lambda: violin(gene_num=int(gene_num.value)))
        
        gene_list = MultiChoice(title='Choose genes:',options=total_gene, value=[''])
        certain_gene = Button(label='Compare genes acros clusters', button_type='success')
        certain_gene.on_click(lambda: compare(gene_list=gene_list.value,group=group))

        self.de = column(row(method,gene_num),
                        rank_genes,
                        violin_figure,
                        gene_list,
                        certain_gene)

    def add(self):
        return self.de


def button_disabled(buttons_group):
    for b in buttons_group:
        b.disabled = True

def button_abled(buttons_group):
    for b in buttons_group:
        b.disabled = False


def get_attr():
    global buttons_group

    plot = plot_function()
    buttons_group, b = plot.get_buttons_group()
    button_disabled(buttons_group)
    def next_get():
        api = connection()
        to_json = api.get_attributes()
        data_dict = json.loads(to_json)
        group = data_dict['selected_group']
        cluster_list = api.get_group_dict()[group]
        return group, cluster_list
        button_abled(buttons_group)
    curdoc().add_next_tick_callback(next_get)


def rank(method,gene_num):
    button_disabled(buttons_group)
    def next_rank():
        layout = curdoc().get_model_by_name('Differential_Expression_Analysis')
        api = connection()
        to_json = api.get_attributes()
        data_dict = json.loads(to_json)
        group = data_dict['selected_group']
        cluster_list = list(api.get_group_dict()[group]['class_name'])
        adata = api.get_anndata()
        print(cluster_list)
        sc.tl.rank_genes_groups(adata, group, groups=cluster_list[1:], reference=cluster_list[0], method=method)
        sc.pl.rank_genes_groups(adata, groups=cluster_list[1:], n_genes=gene_num, save='.png')

        name = 'figures/rank_genes_groups_' + group + '.png'
        img = open(name,'rb')
        img_base64 = base64.b64encode(img.read()).decode("ascii")
        div = Div(text="<img src=\'data:image/png;base64,{}\'/>".format(img_base64))
        layout.children.append(div)
        api.set_uns(adata.uns)
        button_abled(buttons_group)
    curdoc().add_next_tick_callback(next_rank)

def violin(gene_num):
    button_disabled(buttons_group)
    def next_violin():
        layout = curdoc().get_model_by_name('Differential_Expression_Analysis')
        api = connection()
        to_json = api.get_attributes()
        data_dict = json.loads(to_json)
        group = data_dict['selected_group']
        cluster_list = list(api.get_group_dict()[group]['class_name'])
        adata = api.get_anndata()

        sc.pl.rank_genes_groups_violin(adata, groups=cluster_list[1:], n_genes=gene_num, save='.png')

        for cluster in cluster_list[1:]:
            name = 'figures/rank_genes_groups_' + group + '_' + cluster + '.png'
            img = open(name,'rb')
            img_base64 = base64.b64encode(img.read()).decode("ascii")
            div = Div(text="<img src=\'data:image/png;base64,{}\'/>".format(img_base64))
            layout.children.append(div)
        button_abled(buttons_group)
    curdoc().add_next_tick_callback(next_violin)

def compare(gene_list,group):
    button_disabled(buttons_group)
    def next_compare():
        layout = curdoc().get_model_by_name('Differential_Expression_Analysis')
        api = connection()

        adata = api.get_anndata()

        sc.pl.violin(adata, gene_list, groupby=group, save='.png')

        img = open('figures/violin.png','rb')
        img_base64 = base64.b64encode(img.read()).decode("ascii")
        div = Div(text="<img src=\'data:image/png;base64,{}\'/>".format(img_base64))
        layout.children.append(div)
        button_abled(buttons_group)
    curdoc().add_next_tick_callback(next_compare)


import os
import scanpy as sc
from io import BytesIO
from bokeh.palettes import d3
import json
import base64
import sys
from pathlib import Path
from bokeh.io import curdoc
from bokeh.models import FileInput, Button, TextInput, Div, Select
from bokeh.layouts import row, column

sys.path.append(str(Path(__file__).resolve().parents[1]))
try:
    from source import connection, plot_function
except:
    from scpantheon.source import connection, plot_function

color_list = d3['Category20c'][20]


class new_layout:
    def __init__(self):
        sc_cluster_step1 = Button(label='Step1: Run PCA', button_type='success')
        sc_cluster_step1.on_click(pca)
        cl_input1 = TextInput(title='Neighbor Num:', value='10')
        cl_input2 = TextInput(title='Principal Component Num:', value='40')
        cl_input3 = TextInput(title='Resolution', value='1')
        sc_cluster_step2 = Button(label='Step2: Clustering with Neighborhood Graph', button_type='success')
        sc_cluster_step2.on_click(lambda: neighborhood_graph(cl_input1.value, cl_input2.value, cl_input3.value))
        
        # layout format changed: delete the inner column
        self.scanpy_cluster = column(sc_cluster_step1,
                                     row(cl_input1, cl_input2 ,cl_input3),
                                     sc_cluster_step2)
        # self.scanpy_cluster.name = 'Clustering_with_Scanpy'

    def add(self):
        return self.scanpy_cluster


def button_disabled(buttons_group):
    for b in buttons_group:
        b.disabled = True

def button_abled(buttons_group):
    for b in buttons_group:
        b.disabled = False
    

def pca():
    global buttons_group
    plot = plot_function()
    buttons_group, b = plot.get_buttons_group()
    button_disabled(buttons_group)
    def next_pca():
        layout = curdoc().get_model_by_name('Clustering_with_Scanpy')
        change = connection()
        adata = change.get_anndata()

        sc.tl.pca(adata, svd_solver='arpack')
        sc.pl.pca_variance_ratio(adata, log=True, save='.png')

        img = open('figures/pca_variance_ratio.png','rb')
        img_base64 = base64.b64encode(img.read()).decode("ascii")
        div = Div(text="<img src=\'data:image/png;base64,{}\'/>".format(img_base64))
        layout.children.append(div)
        change.set_obsm(adata.obsm)
        button_abled(buttons_group)
    curdoc().add_next_tick_callback(next_pca)

def neighborhood_graph(neighbor_num, pc_num, resolution):
    button_disabled(buttons_group)
    def next_neighbor():
        layout = curdoc().get_model_by_name('Clustering_with_Scanpy')
        change = connection()
        adata = change.get_anndata()

        adata = sc.pp.neighbors(adata, n_neighbors=int(neighbor_num), n_pcs=int(pc_num), copy=True)
        sc.tl.umap(adata)
        sc.tl.leiden(adata, resolution=float(resolution))
        print('new')

        change.set_obsm(adata.obsm)
        change.set_uns(adata.uns)
        change.set_obs(adata.obs, set_group_name=['leiden'])
        button_abled(buttons_group)
    curdoc().add_next_tick_callback(next_neighbor)


from setuptools import setup
import setuptools

# python setup.py sdist bdist_wheel
# python -m twine upload dist/*

setup(
    name='scPANTHEON',# 需要打包的名字,即本模块要发布的名字
    version='0.2.4',#版本
    description='A graphical interface for single cell analysis.', # 简要描述
    packages=['scpantheon'],   #  需要打包的模块
    author='xinzhu', # 作者名
    author_email='xinzhu.jiang@sjtu.edu.cn',   # 作者邮件
    url='https://github.com/xinzhu-email/Pantheon', # 项目地址,一般是代码托管的网站
    install_requires=['bokeh==2.4.3','pandas','anndata==0.8.0','colorcet','scanpy','numpy','PyQt5==5.15.9','PyQtWebEngine==5.15.6','appdirs==1.4.4'], # 依赖包,如果没有,可以不要
    extras_require={
        'tomas': ['tomas'],
    }, # 依赖包,深度使用需手动安装
    entry_points={
        'console_scripts': [
            'sc-pantheon = scpantheon.main:main' # scripts -> multiprocessing
        ]
    },
    package_data={'scpantheon':['extension/*'],},
    license='MIT'
)